th, td {
  border: 1px solid var(--md-typeset-table-color);
}

.md-typeset__table {
  line-height: 1.5;
}

.md-typeset__table table:not([class]) {
  font-size: 0.6rem;
  border-collapse: collapse;
}

.md-typeset__table table:not([class]) td,
.md-typeset__table table:not([class]) th {
  padding: 10px;
}
